local addon, ns = ...
local cfg = ns.cfg
DEBUFF_MAX_DISPLAY = 0

local backdrop_tab = { 
    bgFile = "interface\\AddOns\\wBuffs\\media\\backdrop",  
    edgeFile = "interface\\AddOns\\wBuffs\\media\\backdrop_edge",
    tile = false, tileSize = 0, edgeSize = 5, 
    insets = {left = 5, right = 5, top = 5, bottom = 5,},}
local overlay


local make_backdrop = function(f)
	f:SetFrameLevel(20)
	f:SetPoint("TOPLEFT",-2.5,2.5)
	f:SetPoint("BOTTOMRIGHT",2.5,-2.5)
	f:SetBackdrop(backdrop_tab);
	f:SetBackdropColor(0,0,0,0)
	f:SetBackdropBorderColor(0,0,0,1)
end

-- making frame to hold all buff frame elements
local holder = CreateFrame("Frame", "BuffFrameHolder", UIParent)
holder:SetSize(30,30)
holder:SetPoint(unpack(cfg.BUFFpos))

local PositionTempEnchant = function()
	TemporaryEnchantFrame:ClearAllPoints()
        TemporaryEnchantFrame:SetPoint("TOP","UIParent","BOTTOM", -10, 225) 
end

local function CreateBuffStyle(buff, t)
	if not buff or (buff and buff.styled) then return end
	local bn = buff:GetName()
	local border 	= _G[bn.."Border"]
    local icon 		= _G[bn.."Icon"]
	local duration 	= _G[bn.."Duration"]
	local count 	= _G[bn.."Count"]
	if icon and not _G[bn.."Background"] then
		local h = CreateFrame("Frame")
		h:SetParent(buff)
		h:SetAllPoints(buff)
		h:SetFrameLevel(129)
		icon:SetTexCoord(.08, .92, .08, .92)
		icon:SetPoint("TOPLEFT", buff, 2, -2)
		icon:SetPoint("BOTTOMRIGHT", buff, -2, 2)
		buff:SetSize(cfg.iconsize,cfg.iconsize)
		duration:ClearAllPoints()
		duration:SetParent(h)
		duration:SetPoint("BOTTOM", 0, cfg.timeYoffset)
		duration:SetFont(GameFontNormal:GetFont(), cfg.timefontsize, "THINOUTLINE")
		local bg = CreateFrame("Frame", bn.."Background", buff)
		make_backdrop(bg)
		count:SetParent(h)
		count:ClearAllPoints()
		count:SetPoint("TOPRIGHT")
		count:SetFont(GameFontNormal:GetFont(), cfg.countfontsize, "OUTLINE")
	end
	if border then 
		border:SetTexture("interface\\AddOns\\wBuffs\\media\\iconborder")
		border:SetTexCoord(0.03, 0.97, 0.03, 0.97)
		border:SetPoint("TOPLEFT",2,-2)
		border:SetPoint("BOTTOMRIGHT",-2,2)
		if t == "enchant" then border:SetVertexColor(0.7,0,1) end
	end
	buff.styled = true
end

local function OverrideBuffAnchors()
	local buff, previousBuff, aboveBuff;
	local numBuffs = 0;
	for i=1, BUFF_ACTUAL_DISPLAY do
		--CreateBuffStyle(buttonName, i, false)
		local buff = _G["BuffButton"..i]
		if not buff.styled then CreateBuffStyle(buff) end
		numBuffs = numBuffs + 1
		index = numBuffs
		buff:SetParent(BuffFrame)
		buff.consolidated = nil
		buff.parent = BuffFrame
		buff:ClearAllPoints()
		if ((index > 1) and (mod(index, cfg.BUFFs_per_row) == 1)) then
			buff:SetPoint("TOP", aboveBuff, "BOTTOM", 0, -cfg.spacing*2)
			aboveBuff = buff; 
		elseif ( index == 1 ) then
			local  mh, _, _, oh, _, _, te = GetWeaponEnchantInfo()
			if mh and oh and te and not UnitHasVehicleUI("player") then
			else
				buff:SetPoint("TOPRIGHT", BuffFrame, "TOPRIGHT", 0, 0)
				aboveBuff = buff
			end
			
		else
			buff:SetPoint("LEFT", previousBuff, "RIGHT", cfg.spacing, 0);
		end
		previousBuff = buff
	end
end

local function OverrideTempEnchantAnchors()
	local previousBuff
	for i=1, NUM_TEMP_ENCHANT_FRAMES do
		local te = _G["TempEnchant"..i]
		if te then
			if (i == 1) then
				te:SetPoint("TOPRIGHT", TemporaryEnchantFrame, "TOPRIGHT", 0, 0)
			else
				te:SetPoint("RIGHT", previousBuff, "LEFT", -cfg.spacing, 0)
			end
			previousBuff = te
		end
	end
end

local initialize = function()
local BuffFrame = CreateFrame("frame", "BuffFrame", UIParent)
	BuffFrame:SetScale(cfg.BUFFscale)				--BuffFrame scale
	TemporaryEnchantFrame:SetScale(cfg.BUFFscale)	--temp enchantframe scale
	--position buff & temp enchant frames
	PositionTempEnchant()
	BuffFrame:SetParent(holder)
	BuffFrame:ClearAllPoints()
	BuffFrame:SetPoint(unpack(cfg.BUFFpos))
	--stylize temp enchant frames
	for i=1, NUM_TEMP_ENCHANT_FRAMES do
		local buff = _G["TempEnchant"..i]
		if not buff.styled then CreateBuffStyle(buff, "enchant") end
	end
	OverrideTempEnchantAnchors()
	--getting rid of consolidate buff frame
	if ConsolidatedBuffs then
		ConsolidatedBuffs:UnregisterAllEvents()
		ConsolidatedBuffs:HookScript("OnShow", function(s)
			s:Hide()
			PositionTempEnchant()
		end)
		ConsolidatedBuffs:HookScript("OnHide", function(s)
			PositionTempEnchant()
		end)
		ConsolidatedBuffs:Hide()
	end
end

-- hooking our modifications
hooksecurefunc("BuffFrame_UpdateAllBuffAnchors", OverrideBuffAnchors)
local f = CreateFrame"Frame"
f:RegisterEvent("VARIABLES_LOADED")
f:RegisterEvent("PLAYER_LOGIN")
f:SetScript("OnEvent", function(self, event, ...)
	if event == "VARIABLES_LOADED" then
		SetCVar("consolidateBuffs",0) -- disabling consolidated buffs
		if cfg.disable_timers then cfg.disable_timers = 0 else cfg.disable_timers = 1 end
		SetCVar("buffDurations",cfg.disable_timers) -- enabling buff durations
	else
		initialize()
	end
end)


--[[-----------------------------------------------------------------------------
Movers
-------------------------------------------------------------------------------]]
local silhouette, movers = UIParent:CreateTexture(nil, 'BACKGROUND')
silhouette:SetTexture([[Interface\BUTTONS\WHITE8X8]])
silhouette:SetAlpha(0.45)
silhouette:Hide()

local function GetRelativePoint(frame)
	local L1, B1, W1, H1 = UIParent:GetRect()
	local L2, B2, W2, H2 = frame:GetRect()
	local x, y, h, w, anchor = L2 - L1, B2 - B1, H1 - H2, W1 - W2
	if x < w * 0.5 then
		anchor = 'LEFT'
	else
		anchor, x = 'RIGHT', x - w
	end
	if y < h * 0.5 then
		anchor = 'BOTTOM' .. anchor
	else
		anchor, y = 'TOP' .. anchor, y - h
	end
	return anchor, x, y
end

local function OnMouseDown(self, button)
	if button ~= 'LeftButton' then return end
	self.moving = true
	if not self[4]:IsVisible() or self[5] then
		silhouette:SetParent(self)
		silhouette:ClearAllPoints()
		silhouette:SetAllPoints(self[4])
		silhouette:Show()
	end
	self:StartMoving()
end

local function OnMouseUp(self, button)
	if button ~= 'LeftButton' or not self.moving then return end
	self.moving = nil
	silhouette:Hide()
	self:StopMovingOrSizing()
end

local function CreateMoverFrame(name, moveFrame, silhouette)
	local frame = CreateFrame('Frame', addon .. name .. "Mover", UIParent)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	frame:SetScript('OnMouseDown', OnMouseDown)
	frame:SetScript('OnMouseUp', OnMouseUp)
	frame:Hide()

	local texture = frame:CreateTexture(nil, 'BORDER')
	texture:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
	texture:SetAllPoints()
	texture:SetTexCoord(0, 0.625, 0, 0.6875)

	local string = frame:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	string:SetPoint('CENTER', 1, 0)
	string:SetText(name)
	frame:SetHeight(string:GetStringHeight() + 8)
	frame:SetWidth(string:GetStringWidth() + 16)

	frame:SetPoint('TOP', moveFrame)
	local anchor, x, y = GetRelativePoint(frame)
	frame:ClearAllPoints()
	frame:SetPoint(anchor, x, y)

	moveFrame:ClearAllPoints()
	moveFrame:SetPoint('TOP', frame)
	moveFrame.ClearAllPoints = DoNothing
	moveFrame.SetAllPoints = DoNothing
	moveFrame.SetPoint = DoNothing

	frame[1], frame[2], frame[3], frame[4], frame[5], frame[6] = anchor, x, y, silhouette or moveFrame, silhouette == false or nil
	return frame
end

movers = {
	CreateMoverFrame("Buff", BuffFrame),
}

--[[-----------------------------------------------------------------------------
Slash command
-------------------------------------------------------------------------------]]
_G['SLASH_' .. addon .. 1] = strlower("/wbuff")
SlashCmdList[addon] = function(msg)
	msg = msg:lower()
	if msg == "lock" then
		for index = 1, #movers do
			movers[index]:Hide()
		end
	elseif msg == "reset" then
		local frame
		for index = 1, #movers do
			frame = movers[index]
			frame:ClearAllPoints()
			frame:SetPoint(frame[1], frame[2], frame[3])
		end
	elseif msg == "unlock" then
		for index = 1, #movers do
			movers[index]:Show()
		end
	else
		print("|cff3399ffw|rBuff commands:")
		print("|cff1eff1elock|r - Hide frame mover to prevent dragging.")
		print("|cff1eff1ereset|r - Reset frame to its default position.")
		print("|cff1eff1eunlock|r - Show frame mover to enable dragging.")
	end
end

print("|cff3399ffw|rBuff has been loaded, use /wbuff for help.")